/*++ 
 * linux/drivers/video/wmt/hw/wmt-vpu-reg.h
 * WonderMedia video post processor (VPP) driver
 *
 * Copyright c 2010  WonderMedia  Technologies, Inc.
 *
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * WonderMedia Technologies, Inc.
 * 4F, 533, Chung-Cheng Road, Hsin-Tien, Taipei 231, R.O.C
--*/

/* Product ID: 8650 / Project ID: 3465 */

#ifndef WMT_VPU_REG_H
#define WMT_VPU_REG_H

#include "wmt_mmap.h"
// feature
#define WMT_FTBLK_VPU
#define WMT_FTBLK_VPU_BILINEAR
#define WMT_FTBLK_VPU_RECTAB_EXT

// constant
#define WMT_VPU_RCYC_MIN	1	// 2T
#ifdef WMT_FTBLK_VPU_RECTAB_EXT
#define WMT_VPU_H_DIV_MAX	8192
#define WMT_VPU_V_DIV_MAX	8192
#else
#define WMT_VPU_H_DIV_MAX	256
#define WMT_VPU_V_DIV_MAX	256
#endif

// registers

#define REG_VPU_EN				(VPU_BASE_ADDR+0x00)
#define REG_VPU_UPD				(VPU_BASE_ADDR+0x04)
#define REG_VPU_SEL				(VPU_BASE_ADDR+0x08)

// VPU SCALE
#define REG_VPU_HTB0			(VPU_BASE_ADDR+0x0C)
#define REG_VPU_HTB1			(VPU_BASE_ADDR+0x10)
#define REG_VPU_HPTR			(VPU_BASE_ADDR+0x14)
#define REG_VPU_HRES_TB			(VPU_BASE_ADDR+0x18)
#define REG_VPU_HDIV_TB0		(VPU_BASE_ADDR+0x1C)
#define REG_VPU_HDIV_TB1		(VPU_BASE_ADDR+0x20)
#define REG_VPU_HDIV_TB2		(VPU_BASE_ADDR+0x24)
#define REG_VPU_HDIV_TB3		(VPU_BASE_ADDR+0x28)
#define REG_VPU_HDIV_TB4		(VPU_BASE_ADDR+0x2C)
#define REG_VPU_HDIV_TB5		(VPU_BASE_ADDR+0x30)
#define REG_VPU_HDIV_TB6		(VPU_BASE_ADDR+0x34)
#define REG_VPU_HDIV_TB7		(VPU_BASE_ADDR+0x38)
#define REG_VPU_HWIDTH			(VPU_BASE_ADDR+0x3C)

#define REG_VPU_VTB0			(VPU_BASE_ADDR+0x40)
#define REG_VPU_VTB1			(VPU_BASE_ADDR+0x44)
#define REG_VPU_VPTR			(VPU_BASE_ADDR+0x48)
#define REG_VPU_VRES_TB			(VPU_BASE_ADDR+0x4C)
#define REG_VPU_VDIV_TB0		(VPU_BASE_ADDR+0x50)
#define REG_VPU_VDIV_TB1		(VPU_BASE_ADDR+0x54)
#define REG_VPU_VDIV_TB2		(VPU_BASE_ADDR+0x58)
#define REG_VPU_VDIV_TB3		(VPU_BASE_ADDR+0x5C)
#define REG_VPU_VDIV_TB4		(VPU_BASE_ADDR+0x60)
#define REG_VPU_VDIV_TB5		(VPU_BASE_ADDR+0x64)
#define REG_VPU_VDIV_TB6		(VPU_BASE_ADDR+0x68)
#define REG_VPU_VDIV_TB7		(VPU_BASE_ADDR+0x6C)
#define REG_VPU_VWIDTH			(VPU_BASE_ADDR+0x70)

#define REG_VPU_UPEN			(VPU_BASE_ADDR+0x74)
#define REG_VPU_VSCALE1			(VPU_BASE_ADDR+0x78)
#define REG_VPU_VSCALE2			(VPU_BASE_ADDR+0x7C)
#define REG_VPU_VSCALE3			(VPU_BASE_ADDR+0x80)
#define REG_VPU_HSCALE1			(VPU_BASE_ADDR+0x84)
#define REG_VPU_HSCALE2			(VPU_BASE_ADDR+0x88)
#define REG_VPU_HSCALE3			(VPU_BASE_ADDR+0x8C)

#define REG_VPU_R_CROP2			(VPU_BASE_ADDR+0x90)
#define REG_VPU_DROP_LINE		(VPU_BASE_ADDR+0x94)
#define REG_VPU_3426_TYPE_HSYNC	(VPU_BASE_ADDR+0x98)
#define REG_VPU_V_BYPASS		(VPU_BASE_ADDR+0x9C)

// VPU TG
#define REG_VPU_TG_CTL			(VPU_BASE_ADDR+0xA0)
#define REG_VPU_TG_TOTAL		(VPU_BASE_ADDR+0xA4)
#define REG_VPU_TG_V_ACT		(VPU_BASE_ADDR+0xA8)
#define REG_VPU_TG_H_ACT		(VPU_BASE_ADDR+0xAC)
#define REG_VPU_TG_VBI			(VPU_BASE_ADDR+0xB0)
#define REG_VPU_TG_WAITRDY		(VPU_BASE_ADDR+0xB4)
#define REG_VPU_TG_STATUS		(VPU_BASE_ADDR+0xB8)
#define REG_VPU_TG_GOVW			(VPU_BASE_ADDR+0xBC)

// VPU R MIF
#define REG_VPU_R_CTL			(VPU_BASE_ADDR+0xC0)
#define REG_VPU_R_Y1SA			(VPU_BASE_ADDR+0xC4)
#define REG_VPU_R_C1SA			(VPU_BASE_ADDR+0xC8)
#define REG_VPU_R_HSIZE			(VPU_BASE_ADDR+0xCC)
#define REG_VPU_R_CROP			(VPU_BASE_ADDR+0xD0)
#define REG_VPU_R_FF_CTL		(VPU_BASE_ADDR+0xD4)
#define REG_VPU_R_Y2SA			(VPU_BASE_ADDR+0xD8)
#define REG_VPU_R_C2SA			(VPU_BASE_ADDR+0xDC)

// VPU W MIF
#define REG_VPU_W_CTL			(VPU_BASE_ADDR+0xE0)
#define REG_VPU_W_Y1SA			(VPU_BASE_ADDR+0xE4)
#define REG_VPU_W_C1SA			(VPU_BASE_ADDR+0xE8)
#define REG_VPU_W_Y_TIME		(VPU_BASE_ADDR+0xEC)
#define REG_VPU_W_C_TIME		(VPU_BASE_ADDR+0xF0)
#define REG_VPU_W_FF_CTL		(VPU_BASE_ADDR+0xF4)
#define REG_VPU_REC_BIL_SCL_DN	(VPU_BASE_ADDR+0xFC)

// VPU INT
#define REG_VPU_INT_EN			(VPU_BASE_ADDR+0xF8)

// VPU Base2
// VPU CSC
#define REG_VPU_CSC				(VPU_BASE2_ADDR+0x00)
#define REG_VPU_CSC1			(VPU_BASE2_ADDR+0x04)
#define REG_VPU_CSC2			(VPU_BASE2_ADDR+0x08)
#define REG_VPU_CSC3			(VPU_BASE2_ADDR+0x0C)
#define REG_VPU_CSC4			(VPU_BASE2_ADDR+0x10)
#define REG_VPU_CSC5			(VPU_BASE2_ADDR+0x14)
#define REG_VPU_CSC6			(VPU_BASE2_ADDR+0x18)

// VPU DEINTERLACE
#define REG_DEI_EN				(VPU_BASE2_ADDR+0x1C)
#define REG_DEI_RANGE_A			(VPU_BASE2_ADDR+0x20)
#define REG_DEI_RANGE_B			(VPU_BASE2_ADDR+0x24)
#define REG_DEI_RANGE_C			(VPU_BASE2_ADDR+0x28)
#define REG_DEI_RANGE_D			(VPU_BASE2_ADDR+0x2C)
#define REG_DEI_DIFF_YSUM		(VPU_BASE2_ADDR+0x30)
#define REG_DEI_DIFF_USUM		(VPU_BASE2_ADDR+0x34)
#define REG_DEI_DIFF_VSUM		(VPU_BASE2_ADDR+0x38)

// VPU MV
#define REG_MVR_YSA				(VPU_BASE2_ADDR+0x3C)
#define REG_MVR_HSIZE			(VPU_BASE2_ADDR+0x40)

#define REG_SW_FIELD			(VPU_BASE2_ADDR+0x44)

#define REG_VPU_SRAM_CFG		(VPU_BASE2_ADDR+0x48)
#define REG_VPU_HDIV_TB8		(VPU_BASE2_ADDR+0x50)
#define REG_VPU_HDIV_TB9		(VPU_BASE2_ADDR+0x54)
#define REG_VPU_HDIV_TB10		(VPU_BASE2_ADDR+0x58)
#define REG_VPU_HDIV_TB11		(VPU_BASE2_ADDR+0x5C)
#define REG_VPU_HDIV_TB12		(VPU_BASE2_ADDR+0x60)
#define REG_VPU_HDIV_TB13		(VPU_BASE2_ADDR+0x64)
#define REG_VPU_HDIV_TB14		(VPU_BASE2_ADDR+0x68)
#define REG_VPU_HDIV_TB15		(VPU_BASE2_ADDR+0x6C)

#define REG_VPU_VDIV_TB8		(VPU_BASE2_ADDR+0x70)
#define REG_VPU_VDIV_TB9		(VPU_BASE2_ADDR+0x74)
#define REG_VPU_VDIV_TB10		(VPU_BASE2_ADDR+0x78)
#define REG_VPU_VDIV_TB11		(VPU_BASE2_ADDR+0x7C)
#define REG_VPU_VDIV_TB12		(VPU_BASE2_ADDR+0x80)
#define REG_VPU_VDIV_TB13		(VPU_BASE2_ADDR+0x84)
#define REG_VPU_VDIV_TB14		(VPU_BASE2_ADDR+0x88)
#define REG_VPU_VDIV_TB15		(VPU_BASE2_ADDR+0x8C)

#define REG_VPU_BILINEAR_MODE	(VPU_BASE2_ADDR+0x90)
#define REG_VPU_IGS				(VPU_BASE2_ADDR+0x94)
#define REG_VPU_ALPHA			(VPU_BASE2_ADDR+0x98)

/*----------------------- VPU MIF --------------------------------------*/
// REG_VPU_EN,0x0
#define VPU_SCALAR_ENABLE		REG_VPU_EN,BIT0,0

// REG_VPU_UPD,0x4
#define VPU_REG_UPDATE			REG_VPU_UPD,BIT0,0

// REG_VPU_SEL,0x8
#define VPU_REG_READ_SEL		REG_VPU_SEL,BIT0,0

/*----------------------- VPU SCALE --------------------------------------*/
// REG_HPTR,0x14
#define VPU_HPTR_INIT			REG_VPU_HPTR,0x1F0000,16
#define VPU_HPTR_RETURN			REG_VPU_HPTR,0x1F,0

// REG_HXWIDTH,0x3C
#define VPU_HXWIDTH				REG_VPU_HWIDTH,0x1FFF,0

// REG_VPU_VWIDTH,0x70
#define VPU_DST_VXWIDTH			REG_VPU_VWIDTH,0x1FFF0000,16
#define VPU_VXWIDTH				REG_VPU_VWIDTH,0x1FFF,0

// REG_VPU_UPEN,0x74
#define VPU_V_SCALEUP			REG_VPU_UPEN,0x10000,16
#define VPU_H_SCALEUP			REG_VPU_UPEN,0x1,0

// REG_VPU_VSCALE1,0x78
#define VPU_V_SUBSTEP			REG_VPU_VSCALE1,0x1FFF0000,16
#define VPU_V_THR				REG_VPU_VSCALE1,0x1FFF,0

// REG_VPU_VSCALE2,0x7C
#define VPU_V_STEP				REG_VPU_VSCALE2,0x1FFF0000,16
#define VPU_V_I_SUBSTEPCNT		REG_VPU_VSCALE2,0x1FFF,0

// REG_VPU_VSCALE3,0x80
#define VPU_V_I_STEPCNT			REG_VPU_VSCALE3,0x1FFFF,0

// REG_VPU_HSCALE1,0x84
#define VPU_H_SUBSTEP			REG_VPU_HSCALE1,0x1FFF0000,16
#define VPU_H_THR				REG_VPU_HSCALE1,0x1FFF,0

// REG_VPU_HSCALE2,0x88
#define VPU_H_STEP				REG_VPU_HSCALE2,0x1FFF0000,16
#define VPU_H_I_SUBSTEPCNT		REG_VPU_HSCALE2,0x1FFF,0

// REG_VPU_HSCALE3,0x8C
#define VPU_H_I_STEPCNT			REG_VPU_HSCALE3,0x1FFFF,0

// REG_VPU_R_CROP2,0x90
#define VPU_R_VCROP2			REG_VPU_R_CROP2, 0x1FFF0000, 16
#define VPU_R_HCROP2			REG_VPU_R_CROP2, 0x1FFF, 0

// REG_VPU_DROP_LINE,0x94
#define VPU_HBI_MODE			REG_VPU_DROP_LINE,0x300,8	// 0-disable, 1-0~1/2, 3-1/2~1
#define VPU_DROP_LINE			REG_VPU_DROP_LINE,0x1,0

// REG_VPU_3426_TYPE_HSYNC,0x98
#define VPU_FOLLOW_426			REG_VPU_3426_TYPE_HSYNC,BIT0,0

/*----------------------- VPU TG --------------------------------------*/
// REG_VPU_TG_CTL,0xA0
#define VPU_TG_SYNC				REG_VPU_TG_CTL,BIT31,31		// 1-imm, 0-wait vbis
#define VPU_TG_RDCYC			REG_VPU_TG_CTL,0xFF0000, 16
#define VPU_TG_WAITRDY_ENABLE	REG_VPU_TG_CTL,BIT8,8
#define VPU_TG_ERR_OFF			REG_VPU_TG_CTL,BIT4,4
#define VPU_TG_ENABLE			REG_VPU_TG_CTL,BIT0,0

// REG_VPU_TG_TOTAL,0xA4
#define VPU_TG_V_ALLLINE		REG_VPU_TG_TOTAL, 0x1FFF0000, 16
#define VPU_TG_H_ALLPIXEL		REG_VPU_TG_TOTAL, 0x1FFF, 0

// REG_VPU_TG_V_ACTIVE,0xA8
#define VPU_TG_V_ACTEND			REG_VPU_TG_V_ACT, 0x1FFF0000, 16
#define VPU_TG_V_ACTBG			REG_VPU_TG_V_ACT, 0xFF, 0
 
// REG_VPU_TG_H_ACTIVE,0xAC
#define VPU_TG_H_ACTEND			REG_VPU_TG_H_ACT, 0x1FFF0000, 16
#define VPU_TG_H_ACTBG			REG_VPU_TG_H_ACT, 0x3FF, 0

// REG_VPU_TG_VBI,0xB0
#define VPU_TG_PVBI				REG_VPU_TG_VBI, 0x1F00, 8
#define VPU_TG_VBIE				REG_VPU_TG_VBI, 0x7F, 0

// REG_VPU_TG_WAITRDY,0xB4
#define VPU_TG_WAITRDY_CNT		REG_VPU_TG_WAITRDY, 0xFFFFFFFF, 0

// REG_VPU_TG_STATUS,0xB8
#define VPU_TG_VBIS_STS			REG_VPU_TG_STATUS,BIT1,1	// vbis set to 1, write 1 clear
#define VPU_INTSTS_TGERR		REG_VPU_TG_STATUS,BIT0,0

// REG_VPU_TG_GOVW,0xBC
#define VPU_TG_GOVWTG_ENABLE	REG_VPU_TG_GOVW,BIT0,0

/*----------------------- VPU R MIF --------------------------------------*/
// REG_VPU_R_CTL,0xC0
#define VPU_R_COLBAR_INVERSION		REG_VPU_R_CTL,BIT26,26
#define VPU_R_COLBAR_MODE			REG_VPU_R_CTL,BIT25,25
#define VPU_R_COLBAR_ENABLE			REG_VPU_R_CTL,BIT24,24
#define VPU_MVR_SRC_FMT				REG_VPU_R_CTL,BIT17,17
#define VPU_R_FIELD_MODE			REG_VPU_R_CTL,BIT16,16	/*0:Frame, 1:Field */
#define VPU_R_MEDIAFMT_H264			REG_VPU_R_CTL,BIT12,12	/*0:MPEG, 1:H264 */
#define VPU_R_COLFMT_YUV			REG_VPU_R_CTL,0x600, 9	/*0:422,1:420,2:444 */
#define VPU_R_420C_FMT				REG_VPU_R_CTL,BIT8,8	/*0:Frame, 1:Field */
#define VPU_R_COLFMT_RGB			REG_VPU_R_CTL,0x30,4	/*0-YUV,1-565,3-32bpp*/
#define VPU_MVR_MIF_ENABLE			REG_VPU_R_CTL,BIT2,2
#define VPU_R_MIF2_ENABLE			REG_VPU_R_CTL,BIT1,1
#define VPU_R_MIF_ENABLE			REG_VPU_R_CTL,BIT0,0	/*0:Disable, 1:Enable */

// REG_VPU_R_Y1SA,0xC4
// REG_VPU_R_C1SA,0xC8

// REG_VPU_R_HSIZE,0xCC
#define VPU_R_YPXLWID				REG_VPU_R_HSIZE, 0x1FFF0000, 16
#define VPU_R_YBUFWID				REG_VPU_R_HSIZE, 0x1FFF, 0

// REG_VPU_R_CROP,0xD0
#define VPU_R_VCROP					REG_VPU_R_CROP, 0x1FFF0000, 16
#define VPU_R_HCROP					REG_VPU_R_CROP, 0x1FFF, 0

// REG_VPU_R_FF_CTL,0xD4
#define VPU_MVR_MIF_ERR				REG_VPU_R_FF_CTL,BIT10,10
#define VPU_R2_MIF_ERR				REG_VPU_R_FF_CTL,BIT9,9
#define VPU_R_INTSTS_R1MIFERR		REG_VPU_R_FF_CTL,BIT8,8
#define VPU_R_FIFO_THR				REG_VPU_R_FF_CTL, 0xF, 0

// REG_VPU_R_Y2SA,0xD8
// REG_VPU_R_C2SA,0xDC

/*----------------------- VPU W MIF --------------------------------------*/
// VPU W MIF
// REG_VPU_W_CTL,0xE0
#define VPU_MVR_H264_FMT			REG_VPU_W_CTL,BIT10,10
#define VPU_W_COLFMT_RGB			REG_VPU_W_CTL,BIT9,9
#define VPU_W_COLFMT_YUV			REG_VPU_W_CTL,BIT8,8
#define VPU_W_MIF_EN				REG_VPU_W_CTL,BIT0,0

// REG_VPU_W_Y1SA,0xE4
// REG_VPU_W_C1SA,0xE8

// REG_VPU_W_Y_TIME,0xEC
#define VPU_W_YPXLWID				REG_VPU_W_Y_TIME,0xFFF0000,16
#define VPU_W_YBUFWID				REG_VPU_W_Y_TIME,0xFFF,0

// REG_VPU_W_C_TIME,0xF0
#define VPU_W_CPXLWID				REG_VPU_W_C_TIME,0xFFF0000,16
#define VPU_W_CBUFWID				REG_VPU_W_C_TIME,0xFFF,0

// REG_VPU_W_FF_CTL,0xF4
#define VPU_W_MIF_RGBERR			REG_VPU_W_FF_CTL,BIT16,16
#define VPU_W_MIF_YERR				REG_VPU_W_FF_CTL,BIT8,8
#define VPU_W_MIF_CERR				REG_VPU_W_FF_CTL,BIT0,0

/*----------------------- VPU INT --------------------------------------*/
// REG_VPU_INT_EN,0xF8
#define VPU_SCL_TG_VBIS_EN			REG_VPU_INT_EN,BIT17,17
#define VPU_SCL_TG_ERR_EN			REG_VPU_INT_EN,BIT16,16
#define VPU_MVR_MIF_ERR_EN			REG_VPU_INT_EN,BIT10,10
#define VPU_R1_MIF_ERR_EN			REG_VPU_INT_EN,BIT9,9
#define VPU_R2_MIF_ERR_EN			REG_VPU_INT_EN,BIT8,8
#define VPU_W_MIF_RGB_ERR_EN		REG_VPU_INT_EN,BIT2,2
#define VPU_W_MIF_Y_ERR_EN			REG_VPU_INT_EN,BIT1,1
#define VPU_W_MIF_C_ERR_EN			REG_VPU_INT_EN,BIT0,0

// REG_VPU_REC_BIL_SCL_DN,0xFC
#define VPU_BIL_H_DN_ENABLE			REG_VPU_REC_BIL_SCL_DN,BIT8,8
#define VPU_BIL_V_DN_ENABLE			REG_VPU_REC_BIL_SCL_DN,BIT0,0

// VPU BASE2
/*----------------------- VPU CSC --------------------------------------*/
// REG_VPU_CSC,0x00
#define VPU_CSC_EN				REG_VPU_CSC,0x10000,16
#define VPU_Y_SUB_EN			REG_VPU_CSC,0x100,8
#define VPU_CSC_MODE			REG_VPU_CSC,0x1,0		// 0:RGB2YUV, 1:YUV2RGB

/*----------------------- VPU DEINTERLACE --------------------------------------*/
// REG_DEI_EN,0x1C
#define VPU_DEI_DETECT_MODE		REG_DEI_EN,BIT16,16		// 0:detect Y only,1:detect Y/C
#define VPU_DEI_ENABLE			REG_DEI_EN,BIT8,8
#define VPU_DEI_MODE			REG_DEI_EN,0x7,0

// REG_DEI_RANGE_A,0x20
// REG_DEI_RANGE_B,0x24
// REG_DEI_RANGE_C,0x28
// REG_DEI_RANGE_D,0x2C

// REG_DEI_DIFF_YSUM,0x30
#define VPU_DEI_SUM_UPDATE_EN	REG_DEI_DIFF_YSUM,BIT31,31
#define VPU_DEI_Y_SUM			REG_DEI_DIFF_YSUM,0x7FFFFFFF,0

// REG_DEI_DIFF_USUM,0x34
#define VPU_DEI_U_SUM			REG_DEI_DIFF_USUM,0x7FFFFFFF,0

// REG_DEI_DIFF_VSUM,0x38
#define VPU_DEI_V_SUM			REG_DEI_DIFF_VSUM,0x7FFFFFFF,0

/*----------------------- VPU MVR --------------------------------------*/
// REG_MVR_YSA,0x3C

// REG_MVR_HSIZE,0x40
#define VPU_MVR_LNSIZE			REG_MVR_HSIZE,0x1FFF0000,16
#define VPU_MVR_FBW				REG_MVR_HSIZE,0x1FFF,0

// REG_SW_FIELD,0x44
#define VPU_SW_FIELD			REG_SW_FIELD,BIT8,8
#define VPU_SW_FIELD_ENABLE		REG_SW_FIELD,BIT0,0

// REG_VPU_SRAM_CFG,0x48

// REG_VPU_HDIV_TB8,0x50
// REG_VPU_HDIV_TB9,0x54
// REG_VPU_HDIV_TB10,0x58
// REG_VPU_HDIV_TB11,0x5C
// REG_VPU_HDIV_TB12,0x60
// REG_VPU_HDIV_TB13,0x64
// REG_VPU_HDIV_TB14,0x68
// REG_VPU_HDIV_TB15,0x6C

// REG_VPU_VDIV_TB8,0x70
// REG_VPU_VDIV_TB9,0x74
// REG_VPU_VDIV_TB10,0x78
// REG_VPU_VDIV_TB11,0x7C
// REG_VPU_VDIV_TB12,0x80
// REG_VPU_VDIV_TB13,0x84
// REG_VPU_VDIV_TB14,0x88
// REG_VPU_VDIV_TB15,0x8C

// REG_VPU_BILINEAR_MODE,0x90
#define VPU_BILINEAR_V_EN		REG_VPU_BILINEAR_MODE,BIT8,8
#define VPU_BILINEAR_H_EN		REG_VPU_BILINEAR_MODE,BIT0,0

// REG_VPU_IGS,0x94
#define VPU_IGS_MODE			REG_VPU_IGS,0x3,0	// 0-888(4 byte), 1-5515(2 byte), 2-666(4 byte) , 3-565(2 byte)

// REG_VPU_ALPHA,0x98
#define VPU_FIX_ALPHA_DATA		REG_VPU_ALPHA,0xFF00,8
#define VPU_FIX_ALPHA_ENABLE	REG_VPU_ALPHA,BIT0,0

#endif				/* WMT_VPU_REG_H */
